
package org.eclipse.wtp.releng.cvslog.tools;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.xml.utils.StringToIntTable;
import org.eclipse.wtp.releng.cvstools.RevisionDocument;


public class CVSRevision implements Comparable {
	
	public class RevisedFile {

		public String fileName;

		public String revision;

		public String folderName;

		public String action = "unknown";

		protected RevisedFile() {
		}

		public boolean equals(Object o) {
			if (o == null || !(o instanceof RevisedFile)) {
				return false;
			}

			RevisedFile mod = (RevisedFile) o;

			boolean folderNamesAreEqual = (folderName != null) ? folderName
					.equals(mod.folderName) : (mod.folderName == null);

			boolean revisionsAreEqual = (revision != null) ? revision
					.equals(mod.revision) : (mod.revision == null);

			return (action.equals(mod.action) && fileName.equals(mod.fileName)
					&& folderNamesAreEqual && revisionsAreEqual);
		}

		public int hashCode() {
			int code = 1;
			if (fileName != null) {
				code += fileName.hashCode() * 2;
			}
			if (revision != null) {
				code += revision.hashCode() * 3;
			}
			if (folderName != null) {
				code += folderName.hashCode() * 5;
			}
			if (action != null) {
				code += action.hashCode() * 7;
			}
			return code;
		}
	}


	public Date date;
	
	public String stringDate;
	
	public String bugNo;

	public String author;
	
	public String rev;

	public String comment = "";

	public List files = new ArrayList();

	public CVSRevision() {
	}



	public final RevisedFile createModifiedFile(String filename, String folder) {
		RevisedFile file = newModifiedFile();
		file.fileName = filename;
		file.folderName = folder;
		files.add(file);
		return file;
	}

	protected RevisedFile newModifiedFile() {
		return new RevisedFile();
	}
	
	public RevisionDocument.Revision toXML(RevisionDocument.Revision revision)
	{
		
		revision.setComments(this.comment);
		revision.setAuthor(this.author);
		revision.setRev(this.rev);
		if(stringDate != null)
			revision.setDate(stringDate);
		else
			revision.setDate(CVSRlog.formatCVSDate(date));
		
		
		revision.setBugzilla(this.getBugzilla());
		if(this.getBugzilla()!=null){
				
			try{
				Integer.parseInt(getBugzilla());
				
				Map bugzillaData = BugzillaUtil.getBugzillaData(getBugzilla());
				
				revision.setBugzillaAssignedTo((String)bugzillaData.get(BugzillaUtil.BUGZILLA_ASSIGNTO));
				revision.setBugzillaHardware((String)bugzillaData.get(BugzillaUtil.BUGZILLA_HARDWARE));
				revision.setBugzillaOs((String)bugzillaData.get(BugzillaUtil.BUGZILLA_OS));
				revision.setBugzillaPriority((String)bugzillaData.get(BugzillaUtil.BUGZILLA_PRIORITY));
				revision.setBugzillaResolution((String)bugzillaData.get(BugzillaUtil.BUGZILLA_RESOLUTION));
				revision.setBugzillaSeverity((String)bugzillaData.get(BugzillaUtil.BUGZILLA_SEVERITY));
				revision.setBugzillaStatus((String)bugzillaData.get(BugzillaUtil.BUGZILLA_STATUS));
				revision.setBugzillaSummary((String)bugzillaData.get(BugzillaUtil.BUGZILLA_SUMMARY));
				revision.setBugzillaTargetMilestone((String)bugzillaData.get(BugzillaUtil.BUGZILLA_TARGET_MILESTONE));
				revision.setBugzillaVersion((String)bugzillaData.get(BugzillaUtil.BUGZILLA_VERSION));
			}catch (Exception e) {
				//BugzillaUtil.consoleLogBug("NoBugRevision["+revision.getComments()+"]");
			}	
			
		}
		
		return revision;	
	}
	public RevisionDocument toXML()
	{
		RevisionDocument revisionDocument = RevisionDocument.Factory.newInstance();
		toXML(revisionDocument.addNewRevision());
		return revisionDocument;
		
		
	}

	public String getBugzilla()
	{
		if(bugNo != null)
			return bugNo;
		
		else if(comment != null && comment.indexOf('[') >= 0 ){
			int start = comment.indexOf('[');
			int end = comment.indexOf(']');
			String content = "";
			if( start >= 0 && end > start)
				content = comment.substring(start+1,end);
			String[] bugs = content.split(",|\\s");
			for (int i = 0; i < bugs.length; i++) {
				String aBug = bugs[i];
				try{
					Integer.parseInt(aBug);
					return aBug;
				}catch (Exception e) {
				}
			}
		}
		return null;

	}
	
	
	public String toString() {
		return this.toXML().xmlText();
	}


	/**
	 * Convenience method for getting the filename of the first file
	 * 
	 * @return
	 */
	public String getFileName() {
		if (files.isEmpty()) {
			return null;
		} else {
			return ((RevisedFile) files.get(0)).fileName;
		}
	}

	/**
	 * Convenience method for getting the foldername of the first file
	 * 
	 * @return
	 */
	public String getFolderName() {
		if (files.isEmpty()) {
			return null;
		} else {
			return ((RevisedFile) files.get(0)).folderName;
		}
	}

	public int compareTo(Object o) {
		CVSRevision modification = (CVSRevision) o;
		return date.compareTo(modification.date);
	}

	public boolean equals(Object o) {
		if (o == null || !(o instanceof CVSRevision)) {
			return false;
		}

		CVSRevision mod = (CVSRevision) o;

		boolean revisionsAreEqual = (rev != null) ? rev
				.equals(mod.rev) : (mod.rev == null);

		boolean filesAreEqual = files.size() == mod.files.size();
		for (int i = 0; filesAreEqual && i < files.size(); i++) {
			filesAreEqual = mod.files.get(i).equals(files.get(i));
		}

		return (date.equals(mod.date)
				&& author.equals(mod.author) && revisionsAreEqual
				&& comment.equals(mod.comment));
	}

	public int hashCode() {
		int code = 1;
		if (date != null) {
			code += date.getTime();
		}
		if (author != null) {
			code += author.hashCode() * 5;
		}
		
		if (comment != null) {
			code += comment.hashCode() * 11;
		}
		if (rev != null) {
			code += rev.hashCode() * 13;
		}
		if (files != null) {
			code += fileHashComponent();
		}
		return code;
	}

	private int fileHashComponent() {
		int code = 1;
		for (int i = 0; i < files.size(); i++) {
			CVSRevision.RevisedFile file = (RevisedFile) files.get(i);
			code += file.hashCode();
		}
		return code;
	}
}